@@ -3,6 +3,7 @@
 # @Author : mocobk
 # @Email : mailmzb@qq.com
 # @Time : 2021/2/21 18:15
+import time
 from pathlib import Path
 
 import click
@@ -102,10 +103,10 @@ def upload(ctx, file, type: str):
         ctx.exit()
     click.echo(click.style(f'迭代: {iteration_name}', fg='bright_red', bold=True))
     if type.lower() == 'bvt':
+        case_type = '冒烟'
         cases = BVTCases(excel)
     else:
-        click.echo(click.style('上传所有用例仍在加紧在开发中...', fg='bright_red', bold=True))
-        ctx.exit()
+        case_type = '所有'
         cases = Cases(excel)
 
     results, cases_count = cases.validate_data()
@@ -114,8 +115,9 @@ def upload(ctx, file, type: str):
             click.echo(click.style(reason, fg='bright_red', bold=True))
         ctx.exit()
     else:
-        if click.confirm(click.style(f'即将上传用例数 {cases_count} 条， 是否继续？', fg='bright_cyan', bold=True)):
+        if click.confirm(click.style(f'即将上传或更新{case_type}用例数 {cases_count} 条， 是否继续？', fg='bright_cyan', bold=True)):
             cases.upload(CONFIG_DATA.get('tester'))
+            click.echo(click.style('上传成功', fg='bright_cyan', bold=True))
 
             # 因为重新编辑后的 Excel 会丢失数据校验数据，这里上传完后重新加入数据校验
             # openpyxl/worksheet/_reader.py:308:
@@ -126,7 +128,14 @@ def upload(ctx, file, type: str):
                 'iteration_stories': tapd.get_tapd_iteration_stories()
             })
             excel.set_iteration_data_validation()
+            file_name = file
+            try:
+                excel.save(filename=file_name)
+            # 同名保存失败大部分是因为用例文件是打开状态，无权限写入，重新取名保存
+            except Exception:
+                file_name = time.strftime('%Y%m%d%H%M%S') + file
+                excel.save(filename=file_name)
 
-            excel.save(filename=file)
+            click.echo(click.style('用例文件已更新', fg='green', bold=True))
+            click.echo(click.style(str(Path(file_name).absolute()), fg='green', bold=True))
 
-            click.echo(click.style('用例上传成功', fg='bright_cyan', bold=True))
