@@ -73,7 +73,7 @@ class Tapd:
     def get_tapd_iteration_developers(self):
         url = urljoin(self.base_url, '/tapd/origin/tasks')
         response = requests.get(
-            url, params={'iteration_id': self.iteration_id, 'fields': 'owner', 'order': 'owner asc'}
+            url, params={'iteration_id': self.iteration_id, 'fields': 'owner', 'order': 'owner asc', 'limit': 200}
         )
         data = [item['Task']['owner'] for item in response.json().get('data', []) if item['Task']['owner']]
         return set(data)
@@ -91,16 +91,42 @@ class Tapd:
                 response.json().get('data', [])]
         return OrderedDict(data)
 
-    def create_tapd_task(self, **kwargs):
-        """name, description, creator, owner, story_id, iteration_id"""
+    def post_tapd_task(self, **kwargs):
+        """
+        创建或更新任务
+        name, description, creator, owner, story_id, iteration_id
+        """
         url = urljoin(self.base_url, '/tapd/origin/tasks')
         kwargs.setdefault('iteration_id', self.iteration_id)
         kwargs.setdefault('priority', 4)
         kwargs.setdefault('custom_field_one', '冒烟')
         response = requests.post(url, data=kwargs)
-        task_id = response.json().get('data', {}).get('Task', {}).get('id')
+        json_data = response.json()
+        if not json_data.get('data'):
+            raise Exception('用例上传失败, 原因：', json_data.get('data', 'no data'))
+
+        task_id = json_data.get('data', {}).get('Task', {}).get('id')
         return task_id
 
+    def post_tapd_case(self, **kwargs):
+        """
+        创建或更新用例
+        id, category_id, name, precondition, steps, expectation, priority, creator, type
+        custom_field_1 一级模块
+        custom_field_2 二级模块
+        custom_field_5 迭代名称
+        """
+        url = urljoin(self.base_url, '/tapd/origin/tcases')
+        kwargs.setdefault('priority', '中')
+        kwargs.setdefault('type', '功能测试')
+        response = requests.post(url, data=kwargs)
+        json_data = response.json()
+        if not json_data.get('data'):
+            raise Exception('用例上传失败, 原因：', json_data.get('data', 'no data'))
+
+        case_id = json_data.get('data', {}).get('Tcase', {}).get('id')
+        return case_id
+
 
 class Excel:
     def __init__(self, filename, tapd: Tapd = None):
@@ -206,10 +232,70 @@ class Cases:
         self.title_column_letters = self.excel.case_title_column_letters
 
     def validate_data(self):
-        pass
+        results = []
+        cases_count = 0
+        for row_index in range(3, self.case_ws.max_row + 1):
+            case_title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
+            custom_field_1_cell = f'{self.title_column_letters["一级模块"]}{row_index}'
+            if self.case_ws[case_title_cell].value:
+                if not self.case_ws[custom_field_1_cell].value:
+                    results.append(f'第 {row_index} 行用例【一级模块】不能为空')
+                else:
+                    cases_count += 1
+        return results, cases_count
 
     def upload(self, creator):
-        pass
+        # 如果 case_id 列不存在（用例未上传过），则插入 case_id 列至最后，否则代表用例有上传过，不处理
+        if 'case_id' not in self.title_column_letters:
+            self.title_column_letters['case_id'] = self.case_ws.cell(2, self.case_ws.max_column + 1,
+                                                                     'case_id').column_letter
+
+        iteration_name = self.iteration_meta_info.get('iteration_name')
+        categories_map = self.excel.tapd.get_tapd_case_categories()
+
+        def html_format(content: str):
+            return '<br />'.join(content.strip().splitlines())
+
+        # 使用进度条
+        with click.progressbar(
+                iterable=range(3, self.case_ws.max_row + 1),  # 遍历行号
+                label="上传进度",
+                bar_template="%(label)s | %(bar)s | %(info)s",
+                fill_char=click.style("█", fg="green"),
+                empty_char=" ",
+        ) as bar:
+            time.sleep(0.5)  # 强制先暂停 0.5s, 避免太快进度条没刷出来就结束了，体验提升 50%
+            for row_index in bar:
+                """
+                id, category_id, name, precondition, steps, expectation, priority, creator, type
+                custom_field_1 一级模块
+                custom_field_2 二级模块
+                custom_field_5 迭代名称
+                """
+                custom_field_1_cell = f'{self.title_column_letters["一级模块"]}{row_index}'
+                custom_field_2_cell = f'{self.title_column_letters["二级模块"]}{row_index}'
+                title_cell = f'{self.title_column_letters["用例名称"]}{row_index}'
+                precondition_cell = f'{self.title_column_letters["前置条件"]}{row_index}'
+                steps_cell = f'{self.title_column_letters["用例步骤"]}{row_index}'
+                expected_cell = f'{self.title_column_letters["预期结果"]}{row_index}'
+                priority_cell = f'{self.title_column_letters["用例等级"]}{row_index}'
+                case_id_cell = f'{self.title_column_letters["case_id"]}{row_index}'
+                case_id = self.excel.tapd.post_tapd_case(
+                    id=self.case_ws[case_id_cell].value or None,
+                    category_id=categories_map.get(self.case_ws[custom_field_1_cell].value, 'None Category'),
+                    name=self.case_ws[title_cell].value,
+                    precondition=html_format(self.case_ws[precondition_cell].value or ''),
+                    steps=html_format(self.case_ws[steps_cell].value or ''),
+                    expectation=html_format(self.case_ws[expected_cell].value or ''),
+                    priority=html_format(self.case_ws[priority_cell].value or ''),
+                    creator=creator,
+                    custom_field_1=self.case_ws[custom_field_1_cell].value or '',
+                    custom_field_2=self.case_ws[custom_field_2_cell].value or '',
+                    custom_field_5=iteration_name,
+                )
+
+                if case_id:
+                    self.case_ws[case_id_cell].value = case_id
 
 
 class BVTCases(Cases):
@@ -230,14 +316,14 @@ class BVTCases(Cases):
         return results, cases_count
 
     def upload(self, creator):
-        # 插入 task_id 列至最后
-        task_id_cell = self.case_ws.cell(2, self.case_ws.max_column + 1, 'task_id')
+        # 如果 task_id 列不存在（用例未上传过），则插入 task_id 列至最后，否则代表用例有上传过，不处理
+        if 'task_id' not in self.title_column_letters:
+            self.title_column_letters['task_id'] = self.case_ws.cell(2, self.case_ws.max_column + 1,
+                                                                     'task_id').column_letter
 
         template = Template(BVT_TASK_TEMPLATE.read_text(encoding='utf-8'))
         stories_map = self.iteration_meta_info.get('iteration_stories', {})
 
-        # self.excel.tapd.create_tapd_task = lambda *args, **kwargs: time.sleep(1)
-
         # 使用进度条
         with click.progressbar(
                 iterable=range(3, self.case_ws.max_row + 1),  # 遍历行号
@@ -255,7 +341,10 @@ class BVTCases(Cases):
                     steps_cell = f'{self.title_column_letters["用例步骤"]}{row_index}'
                     expected_cell = f'{self.title_column_letters["预期结果"]}{row_index}'
                     story_cell = f'{self.title_column_letters["需求"]}{row_index}'
-                    task_id = self.excel.tapd.create_tapd_task(
+                    task_id_cell = f'{self.title_column_letters["task_id"]}{row_index}'
+
+                    task_id = self.excel.tapd.post_tapd_task(
+                        id=self.case_ws[task_id_cell].value or None,
                         name=f'【冒烟用例】{self.case_ws[title_cell].value}',
                         creator=creator,
                         owner=self.case_ws[developer_cell].value,
@@ -268,4 +357,4 @@ class BVTCases(Cases):
                         )
                     )
                     if task_id:
-                        self.case_ws.cell(row_index, task_id_cell.column, task_id)
+                        self.case_ws[task_id_cell].value = task_id
